; -------------------------------------------------------------------------
; ffbe.asm (Find Files By Extension)
;
; Finds all files within in a path(recursively), including subdirectories
;
;   Usage: ffbe <path>
;
;          f. ex. ffbe c:\
;
;                 to find all files with extension wcard1 on Drive C:
;                 (please set wcard1 in .data to the extension you need)
; -------------------------------------------------------------------------

include \masm32\include\masm32rt.inc

; -------------------------------------------------------------------------
; Prototypen
; -------------------------------------------------------------------------
Find_Dirs   PROTO
Find_Files  PROTO

; -------------------------------------------------------------------------
.data
    wcard       db  "*.*",0             ; wildcard for Find_Dirs
    wcard1      db  "*.asm",0           ; wildcard for Find_Files
;--------------------- Set this to the extension you need, f.ex. *.exe
    
    szCL        db  MAX_PATH dup(0)     ; Kommandozeile
    szWrkDir    db  MAX_PATH dup(0)     ; the working directory
    szBack      dd  "..",0              ; to go back one directory
  
    cDir        dd  0                   ; number of directories searched
    cFil        dd  0                   ; number of files found
    cByt        dq  0                   ; Total bytes
    
    cfilsiz STRUCT
        fsl dd   ?
        fsh dd   ?
    cfilsiz ENDS
  
    cfs         cfilsiz <>
    
; -------------------------------------------------------------------------
; Codesegment
; -------------------------------------------------------------------------
.code
start:
    call main
    inkey
    exit

; -------------------------------------------------------------------------
; Main
; -------------------------------------------------------------------------
main proc

; Get the path-argument from commandline, if none given currentdir is used
    invoke GetCL, 1, ADDR szCL
    .if eax == 1
        invoke SetCurrentDirectory, ADDR szCL
    .endif

    print "  Searching . . .",13,10  
    invoke Find_Dirs
  
    print chr$(13,10)
    print ustr$(cFil), " files found in "
    print ustr$(cDir), " directories, total "
    print uqword$(cByt)
    print " bytes",13,10,13,10
  
  ret
main endp

; -------------------------------------------------------------------------
; Find_Dirs: Verzeichnisse rekursiv durchlaufen
; -------------------------------------------------------------------------
align 4

Find_Dirs proc

    LOCAL   hSearch :DWORD
    LOCAL   wfd     :WIN32_FIND_DATA

align 4  
    invoke GetCurrentDirectory, MAX_PATH, ADDR szWrkDir
    invoke Find_Files
    invoke FindFirstFile, ADDR wcard, ADDR wfd
    .if eax != INVALID_HANDLE_VALUE
        mov hSearch, eax
    .else
        jmp Find_Next_Dir
    .endif
align 4
    .while eax != 0
        mov ebx, wfd.dwFileAttributes
        .if bx & FILE_ATTRIBUTE_DIRECTORY
            .if wfd.cFileName != "."
                add cDir,1
                invoke SetCurrentDirectory, ADDR wfd.cFileName
                .if eax == NULL
                    print "Error on SetCurrentDirectory "
                    print ADDR wfd.cFileName,13,10
                    jmp Find_Next_Dir
                .endif
                invoke Find_Dirs 
            .endif
        .endif
Find_Next_Dir:        
        invoke FindNextFile, hSearch, ADDR wfd
    .endw
    invoke FindClose, hSearch
    .if eax == NULL
        print "Error on FindClose in Proc Find_Dirs "
        print ADDR szWrkDir
        exit
    .endif
align 4    
    invoke SetCurrentDirectory, ADDR szBack
    invoke GetCurrentDirectory, MAX_PATH, ADDR szWrkDir
    ret
Find_Dirs endp

; -------------------------------------------------------------------------
; Find_Files: Find all files with extension wcard1 in current directory
; -------------------------------------------------------------------------
align 4

Find_Files proc

    LOCAL   hSearch :DWORD
    LOCAL   wfd     :WIN32_FIND_DATA
    
    invoke FindFirstFile, ADDR wcard1, ADDR wfd
    .if eax != INVALID_HANDLE_VALUE
        mov hSearch, eax
    .else
        jmp Find_Exit
    .endif
    .while eax != 0
        mov ebx, wfd.dwFileAttributes
        .if !(bx & FILE_ATTRIBUTE_DIRECTORY)
            print ADDR szWrkDir,"\"
            print ADDR wfd.cFileName,13,10
            add cFil, 1
            mov ebx, wfd.nFileSizeLow
            mov cfs.fsl, ebx
            mov ebx, wfd.nFileSizeHigh
            mov cfs.fsh, ebx 
            fild cfs
            fild cByt
            fadd
            fistp cByt
        .endif
        invoke FindNextFile, hSearch, ADDR wfd
    .endw
    invoke FindClose, hSearch
    .if eax == NULL
        print "FindClose Error in "
        print ADDR szWrkDir,13,10
        exit
    .endif
Find_Exit:
    ret
Find_Files endp

end start
